/*
 * Copyright (C) 2000-2004 by ETH Zurich
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. Neither the name of the copyright holders nor the names of
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ETH ZURICH AND CONTRIBUTORS
 * ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
 * FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ETH ZURICH
 * OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS
 * OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED
 * AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
 * THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * For additional information see http://www.btnode.ethz.ch
 *
 */

 /*
 * bt-cmds.h
 *
 * 11.06.2004 Martin Hinz <btnode@hinz.ch>
 */
#include <bt/bt_defs.h>
#include <bt/bt_hci_cmds.h>

#define MAX_INQ_DEV 10
typedef struct _terminal_bt_cmds_s { 
    struct bt_hci_inquiry_result inqres[MAX_INQ_DEV];
    struct bt_hci_cmd_response inq_respond;
    char nr_inq_dev;
    char inq_time;
    char inq_active;
    int bt_addr_buffer[BD_ADDR_LEN];  //for sscanf, integer needed
    char _bt_cmds_name[20];  //to store local or remote name
    struct btstack* bt_stack;
} _terminal_bt_cmds_t;

extern _terminal_bt_cmds_t *_terminal_bt_cmds;


/**
 * 
\file bt-cmds.h
\author Martin Hinz <btnode@hinz.ch>

\brief Provides terminal commands for bluetooth control.

Commands:\n
- addr (to read the local addr)\n
- inquiry (to make an inquiry)\n
- name (to read the local name)\n
- rname (to read the remote name)\n
- con (to connect to a device)\n
- discon (to disconnect a device)\n
- contable (to print the connection table)\n
- roleset (to set the role of a connection)
- version
- features

* @see \link btn-terminal.h btn-terminal.h \endlink
*/
 
// Read local version
void bt_cmds_version_cmd(u_char* arg);

// Read local supported features
void bt_cmds_features_cmd(u_char* arg);
 
 
/*!
 * @brief Registers the commands into the terminal. 
 * 
 * Call bt_cmds_init() first
 * in order to use them.
 */
void bt_cmds_register_cmds(void);

/*!
 * @brief Initializes the commands. 
 * 
 * @param bt_stack Pointer to the bluetooth stack
 */
void bt_cmds_init(struct btstack* bt_stack);
