#include <string.h>
#include <sys/heap.h>
#include <mhop/rpc.h>
#include <terminal/btn-terminal.h>
#include <debug/logging.h>
#include <debug/toolbox.h>
#include <terminal/rpc-cmds.h>
#include <sys/timer.h>

// debug settings
#define LOG_LEVEL 0
#define LOG_CLASS 9
#include <debug/log_set.h>

#define RPC_CMD_TYPE_LOCAL     0
#define RPC_CMD_TYPE_DEST      1
#define RPC_CMD_TYPE_BROADCAST 2

typedef struct _rpc_cmd_s{
    u_char cmd_type;
    bt_addr_t destination;
    u_char ttl;
    u_char str_proc_nr;
    u_char more_pkts;
    char* data;
    u_short to_send;    
    u_short rpc_psm;
} _rpc_cmd_t;

_rpc_cmd_t* _rpc_cmd;

inline u_short rpcs_max_result_size(void){
    return rpc_max_result_size() - 3;
}

void _rpc_handle_str_cmd(char* cmd){
    u_char data_len = strlen(cmd);
    u_char pkt[data_len + 1];
    pkt[0] = _rpc_cmd->str_proc_nr;
    memcpy(pkt+1, cmd, data_len);
    // send query
    DEBUG("sending query to "ADDR_FMT" len=%d\n", 
       ADDR(_rpc_cmd->destination), data_len);
    mhop_cl_send_pkt(pkt, 
                     data_len + 1, 
                     1, 
                     0, 
                     _rpc_cmd->ttl, 
                     _rpc_cmd->destination, 
                     _rpc_cmd->rpc_psm);   
}

void _rpc_cmd_parser(char* arg){
    u_char i;
    char cmd[INPUT_LENGTH];
    unsigned int val = 0;
    unsigned int addr[BD_ADDR_LEN];
    if(sscanf(arg, "%u%2x:%2x:%2x:%2x:%2x:%2x", &val, &addr[5], &addr[4], 
       &addr[3], &addr[2], &addr[1], &addr[0]) == 7) {
        for (i = 0; i < BD_ADDR_LEN; i++) {
            _rpc_cmd->destination[i] = (u_char) addr[i];
        }
        _rpc_cmd->ttl = (u_char)val;
        _rpc_cmd->cmd_type = RPC_CMD_TYPE_DEST;
    } 
    else if(sscanf(arg, "%u", &val) == 1){ 
        _rpc_cmd->ttl = (u_char)val;
        _rpc_cmd->cmd_type = RPC_CMD_TYPE_BROADCAST;        
    }
    else{
        _rpc_cmd->ttl = 0;
        _rpc_cmd->cmd_type = RPC_CMD_TYPE_BROADCAST;        
    }
    tprintf("type cmd: ");
    user_input(stdout, cmd, INPUT_LENGTH);
    btn_terminal_process_cmd(cmd);
    _rpc_cmd->cmd_type = RPC_CMD_TYPE_LOCAL;                
}

void _rpc_scmd_parser(char* arg){
    u_char i;
    char cmd[INPUT_LENGTH];
    unsigned int val = 0;
    unsigned int addr[BD_ADDR_LEN];
    if(sscanf(arg, "%u%2x:%2x:%2x:%2x:%2x:%2x", &val, &addr[5], &addr[4], 
       &addr[3], &addr[2], &addr[1], &addr[0]) == 7) {
        for (i = 0; i < BD_ADDR_LEN; i++) {
            _rpc_cmd->destination[i] = (u_char) addr[i];
        }
        _rpc_cmd->ttl = (u_char)val;
        _rpc_cmd->cmd_type = RPC_CMD_TYPE_DEST;
    } 
    else if(sscanf(arg, "%u", &val) == 1){ 
        _rpc_cmd->ttl = (u_char)val;
        _rpc_cmd->cmd_type = RPC_CMD_TYPE_BROADCAST;
        memcpy(_rpc_cmd->destination, bt_addr_null, BD_ADDR_LEN);        
    }
    else{
        _rpc_cmd->ttl = 0;
        _rpc_cmd->cmd_type = RPC_CMD_TYPE_BROADCAST;        
        memcpy(_rpc_cmd->destination, bt_addr_null, BD_ADDR_LEN);        
    }
    tprintf("type cmd: ");
    user_input(stdout, cmd, INPUT_LENGTH);
    _rpc_handle_str_cmd(cmd);
    _rpc_cmd->cmd_type = RPC_CMD_TYPE_LOCAL;                
}

u_char _rpc_scmd_proc(char* query, u_char query_len, void* arg,
                      u_char* result, u_char* result_len)
{
    u_char error;
    if(!_rpc_cmd->more_pkts){
        // exec cmd and capture result
        if(term_register_thread() != 0) {
            ERROR("log_register_thread failed\n");
        }
        if(term_start_capture() != 0){        
            ERROR("start capture failed\n");
        }
        query[query_len] = '\0';
        error = btn_terminal_process_cmd(query);
        _rpc_cmd->data = term_stop_capture();
        _rpc_cmd->to_send = strlen(_rpc_cmd->data);
        DEBUG("captured length %d\n", _rpc_cmd->to_send);
    }
    
    if(_rpc_cmd->to_send == 0){
        return 0;
    }
    
    result[0] = _rpc_cmd->to_send >> 8;
    result[1] = _rpc_cmd->to_send & 0xff;

    u_short maxlen = rpcs_max_result_size();
       
    if(_rpc_cmd->to_send > maxlen){
        _rpc_cmd->more_pkts = 1;
        memcpy(result+2, _rpc_cmd->data, maxlen);
        result[2+maxlen]  = '\0'; // null terminate string part
        *result_len = maxlen + 3;
        _rpc_cmd->data += maxlen;
        _rpc_cmd->to_send -= maxlen;
        return 1;
    }
    else{
        _rpc_cmd->more_pkts = 0;
        memcpy(result+2, _rpc_cmd->data, _rpc_cmd->to_send);
        result[2+_rpc_cmd->to_send]  = '\0'; // null terminate string part
        *result_len = _rpc_cmd->to_send + 3;
        return 0;
    }    
}

void _rpc_scmd_result(u_char* source_addr, u_char* result, u_char result_len, 
                      void* arg)
{
    u_short length = (u_short) result[0] << 8 | result[1]; 
    tprintf("\n:RPCS %s %d %d\n%s\n", 
         dbg_bt_addr_to_str(source_addr), 
         length,
         result_len-3,
         result+2);
}

void rpc_handle_query(u_char proc_nr, u_char* data, u_char data_len)
{
    switch(_rpc_cmd->cmd_type){
        case RPC_CMD_TYPE_LOCAL:
            rpc_local_query(proc_nr, data, data_len);
            break;
        case RPC_CMD_TYPE_DEST:            
            rpc_send_query(proc_nr, data, data_len, _rpc_cmd->destination, _rpc_cmd->ttl);
            break;
        case RPC_CMD_TYPE_BROADCAST:
            rpc_send_query(proc_nr, data, data_len, bt_addr_null, _rpc_cmd->ttl); 
            break;     
    }
}


void rpc_cmds_init(u_short rpc_psm)
{
    _rpc_cmd = NutHeapAllocClear(sizeof(_rpc_cmd_t));
    _rpc_cmd->cmd_type = RPC_CMD_TYPE_LOCAL;
    _rpc_cmd->rpc_psm = rpc_psm;
    
    rpc_register_proc(&_rpc_cmd->str_proc_nr,
                      _rpc_scmd_proc, NULL,
                      _rpc_scmd_result, NULL);  
    
    btn_terminal_register_cmd("rpc", _rpc_cmd_parser);                          
    btn_terminal_register_cmd("rpcs", _rpc_scmd_parser);                          
}
